module Main where

import Exp1

data Type = Inteiro | Booleano | String | Erro String deriving (Eq,Show)

typeCheck :: Programa -> Type
typeCheck  (ValInt i) = Inteiro
typeCheck  (ValBool b) = Booleano
-- typeCheck  (ValString s) = String

typeCheck  (Binary e1 Plus e2) 
  | typeCheck e1 == Inteiro && typeCheck e2 == Inteiro
  = Inteiro
  | otherwise
  = Erro "somando um inteiro com um booleano"

typeCheck  (Binary e1 Minus e2) 
  | typeCheck e1 == Inteiro && typeCheck e2 == Inteiro
  = Inteiro
       
typeCheck  (Binary e1 And e2) 
  | typeCheck e1 == Booleano && typeCheck e2 == Booleano
  = Booleano

typeCheck  (Binary e1 Or e2) 
  | (typeCheck e1 == Booleano) && (typeCheck e2 == Booleano)
  = Booleano

typeCheck  (Binary e1 EqEq e2) 
  = let t1 = typeCheck e1
        t2 = typeCheck e2
    in if (t1 == t2)
       then Booleano -- ! t1 !!
       else Erro ""

typeCheck  (Unary Not e1) | (typeCheck e1) == Booleano
   = Booleano

typeCheck  other = Erro ""

eval :: Programa -> Expressao
eval (Binary e1 Plus e2) 
  = let (ValInt e1') = eval e1
        (ValInt e2') = eval e2
    in ValInt (e1' + e2')

eval (Binary e1 Minus e2) 
  = let (ValInt e1') = eval e1
        (ValInt e2') = eval e2
    in ValInt (e1' - e2')
       
eval (Binary e1 And e2) 
  = let (ValBool e1') = eval e1
        (ValBool e2') = eval e2
    in ValBool (e1' && e2')

eval (Binary e1 Or e2) 
  = let (ValBool e1') = eval e1
        (ValBool e2') = eval e2
    in ValBool (e1' || e2')

eval (Binary e1 EqEq e2) 
  = let e1' = eval e1
        e2' = eval e2
    in case e1' of
         ValBool b1 -> let (ValBool b2) = e2'
                       in ValBool (b1 == b2)
         ValInt i1 -> let (ValInt i2) = e2'
                      in ValBool (i1 == i2) -- !

eval (Unary Not e1) 
  = let (ValBool e1') = eval e1
    in ValBool (not e1')

eval other = other

main = do { line <- getLine;
           let program = (exp1 . lexer) line
               typeResult = typeCheck program
           in case typeCheck program of
                Erro str -> putStrLn ("Erro na verificacao de tipos: " ++ str)
                Inteiro -> let (ValInt i) = eval program
                           in putStrLn (show i)
                Booleano ->let (ValBool b) = eval program
                           in putStrLn (show b)
          }
