> {
> module Exp1 where
> import Char
> }

First thing to declare is the name of your parser,
and the type of the tokens the parser reads.

> %name exp1
> %tokentype { Token }

The parser will be of type [Token] -> ?, where ? is determined by the
production rules.  Now we declare all the possible tokens:

> %token 
>	int		{ TokenInt $$ }
>	bool		{ TokenBool $$ }
>	string		{ TokenString $$ }
>	'=='		{ TokenEqEq }
>	'++'		{ TokenPlusPlus }
>	'+'		{ TokenPlus }
>	'-'		{ TokenMinus }
>	'not'		{ TokenNot }
>	'length'	{ TokenLen }
>	'and'		{ TokenAnd }
>	'or' 		{ TokenOr }

The left hand side are the names of the terminals or tokens,
and the right hand side is how to pattern match them.

Like yacc, we include %% here, for no real reason.

> %%

Now we have the production rules.

> Programa :: { Expressao }
> Programa : Expressao { $1 }
>
> Expressao :: { Expressao }
> Expressao : Valor { $1 } 
>           | ExpUnaria { $1 }
>           | ExpBinaria { $1 }
>
> Valor :: { Expressao }
> Valor : ValorConcreto { $1 }
> 
>  ValorConcreto :: { Expressao }
>  ValorConcreto : int		{ ValInt $1 }
> 	         | bool		{ ValBool $1 }
> 	      -- | string	{ ValStr $1 }
>
> ExpUnaria :: { Expressao }
> ExpUnaria : 'not' Expressao		{ Unary Not $2 }
>           | 'length' Expressao	{ Unary Len $2 }
>           -- | '-' Expressao		{ Unary UMinus $2 }
> 
> ExpBinaria :: { Expressao }
> ExpBinaria : Expressao '+' Expressao	{ Binary $1 Plus $3 }
>            | Expressao '-' Expressao	{ Binary $1 Minus $3 }
>            | Expressao 'and' Expressao	{ Binary $1 And $3 }
>            | Expressao 'or' Expressao	{ Binary $1 Or $3 }
>            | Expressao '==' Expressao	{ Binary $1 EqEq $3 }
>            | Expressao '++' Expressao	{ Binary $1 PlusPlus $3 }
> 

We are simply returning the parsed data structure !
Now we need some extra code, to support this parser,
and make in complete:

> {

All parsers must declair this function, 
which is called when an error is detected.
Note that currently we do no error recovery.

> happyError :: [Token] -> a
> happyError _ = error ("Parse error\n")

Now we declare the datastructure that we are parsing.

> type Programa = Expressao

> data Expressao  = ValInt Int | ValBool Bool | ValStr String
>                 | Unary UnOp Expressao
>                 | Binary Expressao Op Expressao
>                 deriving Show

> data Op = Plus | Minus | And | Or | EqEq | PlusPlus
>           deriving Show
> data UnOp = UMinus | Not | Len
>             deriving Show

The datastructure for the tokens...

> data Token
>	= TokenMinus
>	| TokenNot
>	| TokenLen
>	| TokenPlus
>	-- | TokenUMinus
>	| TokenAnd
>	| TokenOr
>	| TokenEqEq
>	| TokenPlusPlus
>	| TokenInt Int
>	| TokenBool Bool
>	| TokenString String
>       deriving Show

.. and a simple lexer that returns this datastructure.

> lexer :: String -> [Token]
> lexer [] = []
> lexer (c:cs) | isSpace c = lexer cs
> 	       | isAlpha c = lexVar (c:cs)
>	       | isDigit c = lexNum (c:cs)
> lexer ('+':'+':cs) = TokenPlusPlus : lexer cs
> lexer ('+':cs) = TokenPlus : lexer cs
> lexer ('-':cs) = TokenMinus : lexer cs
> lexer ('=':'=':cs) = TokenEqEq : lexer cs
> lexer (c:cs) = error ("Lexer error in character " ++ c : "\n")

> lexNum cs = TokenInt (read num) : lexer rest
>	where (num,rest) = span isDigit cs

> lexVar cs =
>    case span isAlpha cs of
>	("and",rest) -> TokenAnd : lexer rest
>	("or",rest)  -> TokenOr : lexer rest
>	("not",rest)  -> TokenNot : lexer rest
>	("length",rest)  -> TokenLen : lexer rest
>	("true",rest)  -> TokenBool True : lexer rest
>	("false",rest)  -> TokenBool False : lexer rest
>	(var,rest)   -> error "Lexer Error\n"

To run the program, call this, or use some code
to print it.

> run :: String -> Expressao
> run = exp1 . lexer

Here we test our parser.

> m1 = getLine >>= \ line ->
>      putStr (show (run line))

> m2 = do { line <- getLine;
>          putStr (show (run line))
>         }

> }
